==> undici-1.2.4.tgz: package/benchmarks/index.js <==
'use strict'
const { Writable } = require('stream')
const http = require('http')
const Benchmark = require('benchmark')
const undici = require('..')
const { kEnqueue, kGetNext } = require('../lib/symbols')
const Request = require('../lib/request')

// # Start the h2o server (in h2o repository)
// # Then change the port below to 8080
// h2o -c examples/h2o/h2o.conf
//
// # Alternatively start the Node.js server
// node benchmarks/server.js
//
// # Start the benchmarks
// node benchmarks/index.js

const httpOptions = {
  protocol: 'http:',
  hostname: 'localhost',
  method: 'GET',
  path: '/',
  port: 3009,
  agent: new http.Agent({
    keepAlive: true,
    maxSockets: 100
  })
}

const undiciOptions = {
  path: '/',
  method: 'GET'
}

const pool = undici(`http://${httpOptions.hostname}:${httpOptions.port}`, {
  connections: 100,
  pipelining: 10,
  requestTimeout: 0
})

const suite = new Benchmark.Suite()

suite
  .add('http - keepalive', {
    defer: true,
    fn: deferred => {
      http.get(httpOptions, response => {
        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        response.pipe(stream)
      })
    }
  })
  .add('http - noop', {
    defer: true,
    fn: deferred => {
      http.get(httpOptions, response => {
        response
          .resume()
          .on('end', () => {
            deferred.resolve()
          })
      })
    }
  })
  .add('undici - pipeline', {
    defer: true,
    fn: deferred => {
      pool
        .pipeline(undiciOptions, data => {
          return data.body
        })
        .end()
        .pipe(new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        }))
        .once('finish', () => {
          deferred.resolve()
        })
    }
  })
  .add('undici - request', {
    defer: true,
    fn: deferred => {
      pool.request(undiciOptions, (error, { body }) => {
        if (error) {
          throw error
        }

        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        body.pipe(stream)
      })
    }
  })
  .add('undici - stream', {
    defer: true,
    fn: deferred => {
      pool.stream(undiciOptions, () => {
        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        return stream
      }, error => {
        if (error) {
          throw error
        }
      })
    }
  })
  .add('undici - simple', {
    defer: true,
    fn: deferred => {
      const stream = new Writable({
        write (chunk, encoding, callback) {
          callback()
        }
      })
      stream.once('finish', () => {
        deferred.resolve()
      })
      const client = pool[kGetNext]()
      client[kEnqueue](new SimpleRequest(client, undiciOptions, stream))
    }
  })
  .add('undici - noop', {
    defer: true,
    fn: deferred => {
      const client = pool[kGetNext]()
      client[kEnqueue](new NoopRequest(client, undiciOptions, deferred))
    }
  })
  .on('cycle', event => {
    console.log(String(event.target))
  })
  .run()

class NoopRequest extends Request {
  constructor (client, opts, deferred) {
    super(opts, client)
    this.deferred = deferred
  }

  _onHeaders () {}

  _onBody () {}

  _onComplete () {
    this.deferred.resolve()
  }
}

class SimpleRequest extends Request {
  constructor (client, opts, dst) {
    super(opts, client)
    this.dst = dst
    this.dst.on('drain', () => {
      this.resume()
    })
  }

  _onHeaders (statusCode, headers, resume) {
    this.resume = resume
  }

  _onBody (chunk, offset, length) {
    return this.dst.write(chunk.slice(offset, offset + length))
  }

  _onComplete () {
    this.dst.end()
  }
}

==> undici-1.3.0.tgz: package/benchmarks/index.js <==
'use strict'
const { Writable } = require('stream')
const http = require('http')
const Benchmark = require('benchmark')
const undici = require('..')
const { kGetNext } = require('../lib/symbols')

// # Start the h2o server (in h2o repository)
// # Then change the port below to 8080
// h2o -c examples/h2o/h2o.conf
//
// # Alternatively start the Node.js server
// node benchmarks/server.js
//
// # Start the benchmarks
// node benchmarks/index.js

const httpOptions = {
  protocol: 'http:',
  hostname: 'localhost',
  method: 'GET',
  path: '/',
  port: 3009,
  agent: new http.Agent({
    keepAlive: true,
    maxSockets: 100
  })
}

const undiciOptions = {
  path: '/',
  method: 'GET'
}

const pool = undici(`http://${httpOptions.hostname}:${httpOptions.port}`, {
  connections: 100,
  pipelining: 10,
  requestTimeout: 0
})

const suite = new Benchmark.Suite()

Benchmark.options.minSamples = 200

suite
  .add('http - keepalive', {
    defer: true,
    fn: deferred => {
      http.get(httpOptions, response => {
        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        response.pipe(stream)
      })
    }
  })
  .add('undici - pipeline', {
    defer: true,
    fn: deferred => {
      pool
        .pipeline(undiciOptions, data => {
          return data.body
        })
        .end()
        .pipe(new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        }))
        .once('finish', () => {
          deferred.resolve()
        })
    }
  })
  .add('undici - request', {
    defer: true,
    fn: deferred => {
      pool.request(undiciOptions, (error, { body }) => {
        if (error) {
          throw error
        }

        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        body.pipe(stream)
      })
    }
  })
  .add('undici - stream', {
    defer: true,
    fn: deferred => {
      pool.stream(undiciOptions, () => {
        const stream = new Writable({
          write (chunk, encoding, callback) {
            callback()
          }
        })
        stream.once('finish', () => {
          deferred.resolve()
        })

        return stream
      }, error => {
        if (error) {
          throw error
        }
      })
    }
  })
  .add('undici - dispatch', {
    defer: true,
    fn: deferred => {
      const stream = new Writable({
        write (chunk, encoding, callback) {
          callback()
        }
      })
      stream.once('finish', () => {
        deferred.resolve()
      })
      const client = pool[kGetNext]()
      client.dispatch(undiciOptions, new SimpleRequest(stream))
    }
  })
  .add('undici - noop', {
    defer: true,
    fn: deferred => {
      const client = pool[kGetNext]()
      client.dispatch(undiciOptions, new NoopRequest(deferred))
    }
  })
  .on('cycle', event => {
    console.log(String(event.target))
  })
  .run()

class NoopRequest {
  constructor (deferred) {
    this.deferred = deferred
  }

  _onHeaders () {

  }

  _onData (chunk) {
    return true
  }

  _onComplete (trailers) {
    this.deferred.resolve()
  }
}

class SimpleRequest {
  constructor (dst) {
    this.dst = dst
  }

  _onHeaders (statusCode, headers, resume) {
    this.dst.on('drain', resume)
  }

  _onData (chunk) {
    return this.dst.write(chunk)
  }

  _onComplete () {
    this.dst.end()
  }
}
